package com.poststats.rs.api;

import java.math.BigInteger;

import org.jboss.resteasy.annotations.cache.Cache;
import org.jboss.resteasy.plugins.providers.multipart.MultipartFormDataInput;

import com.poststats.rs.api.model.Photo;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Encoding;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.media.Schema.RequiredMode;
import io.swagger.v3.oas.annotations.media.SchemaProperty;
import io.swagger.v3.oas.annotations.parameters.RequestBody;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.security.RolesAllowed;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;

/**
 * @author brian.long@poststats.com
 */
@Path("/photo")
@Tag(name = "Photo API")
@Produces({MediaType.APPLICATION_JSON, Constants.V1_JSON})
@ApiResponse(responseCode = "200", description = "Success", content = {
        @Content(mediaType = MediaType.APPLICATION_JSON),
        @Content(mediaType = Constants.V1_JSON)
})
@ApiResponse(responseCode = "400", description = "A request parameter is not valid")
@ApiResponse(responseCode = "404", description = "No matching photo was found")
public interface PhotoApi {

	@GET
	@Path("/{photoId}/metadata")
	@Cache(isPrivate = true, maxAge = 60)
	@Operation(
			summary = "Retrieves limited meta-data about a photo.",
			description = "Retreives type, size, and other direct meta-data about the specified photo."
	)
    @ApiResponses({
            @ApiResponse(responseCode = "200", description = "Success"),
            @ApiResponse(responseCode = "404", description = "Not found")
    })
	Photo get(
			@Parameter(description = "A unique identifier for a photo")
			@PathParam("photoId") @NotNull @Positive BigInteger photoId);

	@GET
	@Path("/{photoId}")
	@Produces({"image/png", "image/jpeg"})
	@Cache(maxAge = 864000)
	@Operation(summary = "Retrieves the binary stream of a photo.")
	@ApiResponses({
	        @ApiResponse(responseCode = "200", description = "Success"),
            @ApiResponse(responseCode = "404", description = "Not found")
	})
	Response getStream(
			@Parameter(description = "A unique identifier for a photo")
			@PathParam("photoId") @NotNull @Positive BigInteger photoId);

	@GET
	@Path("/{photoId}/thumbnail")
	@Produces({"image/png", "image/jpeg"})
	@Cache(maxAge = 864000)
	@Operation(summary = "Retrieves the binary stream of the thumbnail of a photo.")
    @ApiResponses({
            @ApiResponse(responseCode = "200", description = "Success"),
            @ApiResponse(responseCode = "404", description = "Not found")
    })
	Response getThumbnailStream(
            @Parameter(description = "A unique identifier for a photo")
			@PathParam("photoId") @NotNull @Positive BigInteger photoId);

    @DELETE
    @Path("/{photoId}")
    @Operation(summary = "Permanently deletes the photo contents, meta-data, and associations to people, courses, and events.")
    @ApiResponses({
            @ApiResponse(responseCode = "200", description = "Success"),
            @ApiResponse(responseCode = "404", description = "Not found")
    })
    @RolesAllowed("admin")
    @SecurityRequirement(name = "basic")
    void remove(
            @Parameter(description = "A unique identifier for a photo")
            @PathParam("photoId") @NotNull @Positive BigInteger photoId);

	@POST
	@Consumes(MediaType.MULTIPART_FORM_DATA)
	@Operation(
			summary = "Stores a photo and associates it to any other object.",
			description = "Auto-crops, scales, and stores a photo and associates it to facilities, courses, persons, events, and event series.",
			requestBody = @RequestBody(content = @Content(
			        mediaType = MediaType.MULTIPART_FORM_DATA,
			        schemaProperties = {
			                @SchemaProperty(name = "file", schema = @Schema(type = "string", format = "base64", requiredMode = RequiredMode.REQUIRED)),
                            @SchemaProperty(name = "photo", schema = @Schema(type = "string", requiredMode = RequiredMode.REQUIRED))
			        },
			        encoding = {
			                @Encoding(name = "file", contentType = "image/png, image/jpeg"),
                            @Encoding(name = "photo", contentType = MediaType.APPLICATION_JSON + ", " + Constants.V1_JSON)
			        }
			)),
			parameters = {
			        @Parameter(name = "photo", description = "JSON meta-data associated with the uploaded photo", content = {
			                @Content(mediaType = MediaType.APPLICATION_JSON),
                            @Content(mediaType = Constants.V1_JSON)
			        }),
			        @Parameter(name = "file", content = {
			                @Content(mediaType = "image/png"),
                            @Content(mediaType = "image/jpeg")
			        })
			}
	)
	@RolesAllowed("member")
	@SecurityRequirement(name = "basic")
	BigInteger uploadPhoto(MultipartFormDataInput multipart);

}
